package com.arm.android.example.ndkmcpkg;

import com.arm.android.example.ndkmcpkg.R;

import android.app.Activity;
import android.graphics.Bitmap;

import android.os.Bundle;
import android.os.SystemClock;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import java.lang.Runtime;
import java.util.concurrent.Semaphore;

public class ndkmcact extends Activity {
	protected static TextView _tv;
	protected static ImageView _imageView;
	protected static Button _parallelButton;
	protected static Button _serialButton;
	protected static int numcores;
	private Semaphore sanitizer;
	
	/* Function declaration for native interface call */
	public native int getNextFrame(int[] destFrame, int width, int height, int numthreads);

	/* Load bundled native library "libnativecode.so" on app start */
    static {
        System.loadLibrary("nativecode");
    }

    /* Here is the function that actually calls out into the native library */ 
    private void render(int numthreads)
    {
    	/* Ensure no more than one button handler attempts to alter the common resources at once */
    	sanitizer.acquireUninterruptibly();
    	
    	int height, width;
    	ImageView iv = _imageView;

    	/* Find out how much screen real estate we have to work with */
    	height = iv.getMeasuredHeight();
    	width = iv.getMeasuredWidth();
    	
    	/* Allocate an int array to hold all of the available pixels */
    	int[] buf = new int[width * height];

    	/* Get start reading for native call measurement */
    	long start = SystemClock.elapsedRealtime();
    	
    	/* Call native library to get a Bitmap formatted int[] to display */
    	getNextFrame(buf, width, height, numthreads);
    	
    	/* Get end reading for native call measurement */
    	long end = SystemClock.elapsedRealtime();
    	
    	/* Convert our int array to a Bitmap object */
    	Bitmap bmp = Bitmap.createBitmap(buf, width, height, Bitmap.Config.ARGB_8888);

    	/* Load our new Bitmap into our ImageView */
    	iv.setImageBitmap(bmp);

    	/* Display processing time */
    	_tv.setText("time=" + (end - start) + " over " + numthreads + " thread(s)");

		/* All done, let next button press in */
		sanitizer.release();
    }
    
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        /* 
         * Look up IDs of our widgets.
         */
        _parallelButton = (Button) findViewById(R.id.button1);
        _serialButton = (Button) findViewById(R.id.button2);
        _imageView = (ImageView) findViewById(R.id.image1);
        _tv = (TextView) findViewById(R.id.footer);
        
        /* Find out how many cores we have to work with, to make informed decisions */
    	numcores = Runtime.getRuntime().availableProcessors();
		_tv.setText("Number of cores: " + numcores);

		/* Set up a semaphore to prevent launching multiple tasks operating on the same resources */
    	sanitizer = new Semaphore(1);
       
        /* Add handler for triggering parallel decode */
        _parallelButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
            	/* Split task over as many threads as we have available cores */
            	render(numcores);
            }
        });
        _serialButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
            	/* Run the whole task on single thread */
            	render(1);
            }
        });
    }
}
